@@ -87,10 +87,10 @@ def option_q(args):
 
     _queryAndPrint(q,fields=fields,systemFields=systemFields,nullFields=nullFields)
 
-def option_checkCatalogs(args):
+def option_cc(args):
     if '-h' in args:
         help = f"""
-        {utils.CGREEN}-checkCatalogs{utils.CEND} 
+        {utils.CGREEN}-cc{utils.CEND} 
             Checks the catalogs. Gets all catalogs, does a getOffers, getOfferDetails, basketwithoutconfig and basket with config
             path: the path to a child product to configure. Path = ... ProductCode ... ProdcutCode
             quantity: the quantity for the object""" 
@@ -181,7 +181,7 @@ def option_o(args):
             q = f"select fields(all) from {chunks[0]} where Id='{chunks[1]}'"
 
         print('pre q:'+q)
-        _queryAndPrint(q)
+        _queryAndPrint(q,systemFields=True,nullFields=True)
         return
         
     like = getParam(args,'-like') if '-like' in args else None    
@@ -285,6 +285,7 @@ def option_logs(args):
     logId = getParam(args,'-logs')    
     last = getParam(args,'-last')  
     limit = getParam(args,'-limit')    
+    level = getParam(args,'-level') 
     loguser = getParam(args,'-loguser') 
 
     if loguser == None:
@@ -307,7 +308,7 @@ def option_logs(args):
         return
 
     connectionInit(args)
-    debugLogs.parseLog(logId,lastN=last)
+    debugLogs.parseLog(logId,lastN=last,level=level)
 
     return
 
