@@ -23,41 +23,44 @@ LOGDATA:    app: {log['Application']}      status: {log['Status']}     location:
     callbody = logLine + callbody
     return log,callbody
 
-def parseLog(logId=None,printLimits=True,userDebug=True,lastN=1,logUserId=None):
+def parseLog(logId=None,filepath=None,printLimits=True,userDebug=True,lastN=1,logUserId=None,level=None):
     body = None
 
-    if logId is not None:
+    if filepath != None:
+        body = file.read(filepath)
+
+    elif logId != None:  #check if already downloaded
         filename = f"{restClient.logFolder()}{logId}.log"
         if file.exists(filename) == True:
             body = file.read(filename)
-       #     body = None
-
-    if body is None:
-        if logId is None:
-            whereUSer = f" where logUserId='{logUserId}' " if logUserId is not None else ''
-            logIds = query.queryFieldList(f"Select Id FROM ApexLog {whereUSer} order by LastModifiedDate desc limit {lastN}")
-            if logIds == None or len(logIds)==0:
-                utils.raiseException(errorCode='NO_LOG',error=f'No logs can be found. ')
-            for logId in logIds:
-                print('___________________________________________________________________________________________________________________________________________________________________')
-                print(f"Parsing log Id {logId}")
-                parseLog(logId)
-            #    input("Click for next")
-            return  
         else:
-            logRecord,body = queryLogData(logId)
+           logRecord,body = queryLogData(logId) 
+
+    else:
+        whereUSer = f" where logUserId='{logUserId}' " if logUserId is not None else ''
+        logIds = query.queryFieldList(f"Select Id FROM ApexLog {whereUSer} order by LastModifiedDate desc limit {lastN}")
+        if logIds == None or len(logIds)==0:
+            utils.raiseException(errorCode='NO_LOG',error=f'No logs can be found. ')
+        for logId in logIds:
+            print('___________________________________________________________________________________________________________________________________________________________________')
+            print(f"Parsing log Id {logId}")
+            parseLog(logId)
+        #    input("Click for next")
+        return  
 
     if body == None or len(body)==0:
         utils.raiseException(errorCode='NO_LOG',error=f'The requested log <{logId}> cannot be found. ')
-    filename = f"{restClient.logFolder()}{logId}.log"
 
-    file.write(filename,body)
-    parse(body,printLimits=printLimits,userDebug=userDebug)
+    if filepath == None:
+        filename = f"{restClient.logFolder()}{logId}.log"
+        file.write(filename,body)
+
+    return parse(body,printLimits=printLimits,userDebug=userDebug,level=level)
 
 def printLogs(logUserId=None,limit=50):
     logs = queryLogRecords(logUserId,limit=limit)
     logs = utils.deleteNulls(logs,systemFields=False)
-    utils.printFormated(logs)
+    utils.printFormated(logs,rename="LogLength%Len:DurationMilliseconds%ms")
 
 def delta(obj,field):
     return obj[field][1] - obj[field][0] if len(obj[field]) > 1 else 0
@@ -132,39 +135,42 @@ def resetContext():
     _context['previousCPUTime'] = 0
     _context['previousIsLimit'] = False
     _context['prevTimes'] = {
-        "0":[0,0]
+        0:[0,0]
     }
+    _context['prevLevel'] = 0
     _context['firstLineIn'] = True
     _context['firstLineOut'] = True
 
-    
-
-def parse(log,printLimits=True,userDebug=True):
-   # log = file.read(filepath=filepath)
+def parse(logData,printLimits=True,userDebug=True,level=None):
     _context['printLimits'] = printLimits
     _context['userDebug'] = userDebug
+    _context['logLevel'] = level
 
-    lines = log.splitlines()
+    lines = logData.splitlines()
 
     ident = 0
 
     SOQL = {}
 
-    #totalQueries = 0
+    onGoingCalls = []
+    methodsList = onGoingCalls
+    constructorList = onGoingCalls
+    codeUnitsList = onGoingCalls
+    constructorList = onGoingCalls
+    dmlList=onGoingCalls
+
+   # methodsList = []
+   # constructorList = []
+   # codeUnitsList = []
+   # constructorList = []
+   # dmlList=[]
 
-    methodsList = []
-    constructorList = []
-    codeUnitsList = []
-    constructorList = []
-    dmlList=[]
     limits = None
 
     exceptionDict = None
 
     limitsDict = {}
 
-    #workflow = {}
-
     debugList = []
     multiLine = None #for multiline commands
 
@@ -186,20 +192,24 @@ def parse(log,printLimits=True,userDebug=True):
                 print()
                 continue
 
-        if len(chunks)>1 and chunks[1] in ['HEAP_ALLOCATE','STATEMENT_EXECUTE','USER_INFO','VARIABLE_SCOPE_BEGIN']:
+        if len(chunks)>1 and chunks[1] in ['HEAP_ALLOCATE','STATEMENT_EXECUTE','VARIABLE_SCOPE_BEGIN']:
             continue
 
         if '|SYSTEM_MODE_EXIT|' in line:
             nop=1
 
         if '|USER_INFO|' in line:
-            setTimes(line)
+            obj = setTimes(line,field='string',value=chunks[4],type='USER_INFO')
+            debugList.append(obj)
+            obj['output'] = obj['string'] 
+
 
         if '|' in line:   #This is a new line always. 
             multiLine = None
         
         if exceptionDict is not None:
             if multiLine != 'EXCEPTION':   #This is a new line without the exeption
+                exceptionDict['output'] = ""
                 debugList.append(exceptionDict.copy())
                 exceptionDict = None
                 multiLine = None
@@ -214,6 +224,20 @@ def parse(log,printLimits=True,userDebug=True):
             _context['exception'] = True
             multiLine = 'EXCEPTION'
 
+
+        if 'EXP_VAR' in _context and _context['EXP_VAR'] == True:
+            if chunks[1] == 'VARIABLE_ASSIGNMENT' and chunks[2] == '[EXTERNAL]':
+                obj = setTimes(line,type='VAR_ASSIGN')
+                obj['type'] = 'VAR_ASSIGN'
+                obj['subType'] = 'EXCEPTION'
+                obj['string'] = chunks[4]
+                obj['apexline'] = chunks[2][1:-1] if chunks[2]!='[EXTERNAL]' else 'EX'
+
+                debugList.append(obj)         
+                obj['output'] = obj['string'] 
+   
+            else:
+                _context['EXP_VAR'] = False
         if '|VARIABLE_ASSIGNMENT|' in line:
             if len(chunks) >= 5:
                 if 'ExecutionException' in chunks[4]:
@@ -224,6 +248,9 @@ def parse(log,printLimits=True,userDebug=True):
                     obj['apexline'] = chunks[2][1:-1] if chunks[2]!='[EXTERNAL]' else 'EX'
 
                     debugList.append(obj)
+                    obj['output'] = obj['string'] 
+
+            _context['EXP_VAR'] = True
  
         if '|LIMIT_USAGE|' in line:
             if '|SOQL|' in line:
@@ -240,6 +267,7 @@ def parse(log,printLimits=True,userDebug=True):
 
             limitsDict = setTimes(line,type='LIMITS')
             limitsDict['limitType'] = limits
+            limitsDict['output'] = limitsDict['limitType']
 
         if limits is not None and line == ' ':
             limits = None
@@ -281,8 +309,33 @@ def parse(log,printLimits=True,userDebug=True):
                 obj['string'] = line
         #    if _context['userDebug'] == True:
             debugList.append(obj)
+            obj['output'] = obj['string'] 
+
+
+        if 'NAMED_CREDENTIAL_' in line:
+            if "REQUEST" in chunks[1]:
+                obj = setTimes(line,field='string',value=chunks[2],type='NAMED_CRD')
+                onGoingCalls.append(obj)
+                increaseIdent()    
+                debugList.append(obj)
+                obj['output'] = obj['string'] 
+
+            if "NAMED_CREDENTIAL_RESPONSE" == chunks[1]:
+                decreaseIdent()
+                obj = delFromList(onGoingCalls,'type','NAMED_CRD')
+                setTimes(line,obj) 
+
+        if 'CALLOUT_RESPONSE' in line:
+            obj = setTimes(line,type='CALLOUT')
+          #  obj['type'] = 'DEBUG'
+          #  obj['subType'] = chunks[3]
+            obj['string'] = chunks[3]
+            obj['apexline'] = chunks[2][1:-1]
+
+            debugList.append(obj)  
+            obj['output'] = obj['string'] 
+ 
 
-    
         if '|WF_RULE_EVAL' in line:
             if 'BEGIN' in chunks[1]:
                 workflow = setTimes(line,field='method',value=chunks[2])
@@ -299,11 +352,12 @@ def parse(log,printLimits=True,userDebug=True):
                 increaseIdent()    
                 debugList.append(obj)
 
-
             if 'EXIT' in chunks[1]:    
                 decreaseIdent()
                 obj = delFromList(constructorList,'method',chunks[5])
                 setTimes(line,obj)   
+                obj['output'] = obj['method'] 
+
          
         if '|CODE_UNIT_' in line:
             if 'STARTED' in chunks[1]:
@@ -318,6 +372,8 @@ def parse(log,printLimits=True,userDebug=True):
                 decreaseIdent()
                 obj = delFromList(codeUnitsList,'method',chunks[2])
                 setTimes(line,obj)
+                obj['output'] = obj['method']
+
 
         if '|DML_' in line:
             if 'BEGIN' in chunks[1]:
@@ -334,6 +390,8 @@ def parse(log,printLimits=True,userDebug=True):
                 obj = delFromList(dmlList,'Id',chunks[2])
                 setTimes(line,obj)
                 decreaseIdent()
+                obj['output'] = f"{obj['OP']} {obj['Type']}" 
+
 
         if 'SOQL_EXECUTE_' in line:
             if 'BEGIN' in chunks[1]:
@@ -348,6 +406,8 @@ def parse(log,printLimits=True,userDebug=True):
                 _context['totalQueries'] = _context['totalQueries'] + 1
                 setTimes(line,SOQL)
                 SOQL['rows'] = chunks[3].split(':')[1]
+                SOQL['output'] = SOQL['query'] 
+
                 if len(SOQL['query']) > 180:
                     if 'from' in SOQL['query'].lower():
                         SOQL['query'] = SOQL['query'].replace('from',"FROM")
@@ -376,6 +436,8 @@ def parse(log,printLimits=True,userDebug=True):
                     increaseIdent()
             else:
                 obj = delFromList(methodsList,'method',method)
+                if obj == None:
+                    obj = delFromList(methodsList,'method',f".{method}",endsWith=True)
 
                 if obj is not None:
                     decreaseIdent()
@@ -383,13 +445,53 @@ def parse(log,printLimits=True,userDebug=True):
 
                 else:
                     obj = setTimes(line,type='NO_ENTRY')
+                    debugList.append(obj)
+
+                obj['output']=obj['method']
+
+    repDebugList = processRepetition(debugList)
+    printDebugList(repDebugList)
 
-    for d in debugList:
-        printParsedLog(d)
     print()
 
+    return debugList
+def printDebugList(debugList):
+    for d in debugList:
+        if '*** getCpuTime() ***' in d['output']:
+            continue
+        printParsedLog(d)    
+
+def processRepetition(debugList):
+    def isRep(repss,x,parsedLine):
+      if parsedLine['type'] != 'METHOD':
+        return False,None
+
+      for reps in repss:
+        delta = reps[1]-reps[0]
+        if x>= reps[0] and x <= reps[0] + len(reps) * delta - 1:
+          if x>= reps[0] and x <= reps[0] + delta-1:
+            if 'output' not in parsedLine:
+                print()
+            parsedLine['output'] = f"{parsedLine['output']}  *** {len(reps)},  {x-reps[0]+1}"
+            if len(parsedLine['timeStamp'])>1:
+                parsedLine['timeStamp'][1] = debugList[x+(len(reps)-1)*delta]['timeStamp'][1]
+            return True,parsedLine
+          else:
+            return True,None
+
+      return False,None
+
+    repss = utils.repeatingSequence(debugList,"output")
+
+    parsedX = []
+    for x,parsedLine in enumerate(debugList):
+        isrepe, obj = isRep(repss,x,parsedLine)
+        if isrepe == True and obj == None:
+            continue
+        parsedX.append(parsedLine)
+
+    return parsedX   
 
- #  utils.printFormated(logsList)
 def getMethod(line):
     chunks = line.split('|')
     if len(chunks) == 4:
@@ -445,10 +547,12 @@ def printParsedLog(d):
 
     CEND    = '\33[0m'
     CRED    = '\33[31m'
-    CWHITE  = '\33[37m'
-    CBLUE   = '\33[34m'
     CGREEN  = '\33[32m'
     CYELLOW = '\33[33m'
+    CBLUE   = '\33[34m'
+    CMAGENTA = '\033[35m'
+    CCYAN   = '\033[36m'
+    CWHITE  = '\33[37m'
 
     Cinit = CWHITE
 
@@ -465,6 +569,9 @@ def printParsedLog(d):
         _context['previousIsLimit'] = False
 
     level = d['ident']
+    if _context['logLevel'] != None:
+        if level > int(_context['logLevel']):
+            return
 
     _type = d['type']
     if _type == 'DEBUG':
@@ -477,23 +584,32 @@ def printParsedLog(d):
         else:
             return
 
+        
     if d['type'] == 'SOQL':
-        Cinit = CBLUE
+        Cinit = CCYAN
     if d['type'] == 'DML':
         Cinit =  CYELLOW
+    if d['type'] == 'CODE_UNIT':
+        Cinit =  CMAGENTA
 
     i = f"{emptyString(1,' ',level)}."
     i= level
     identation = f"{emptyString(3,' ',level)}"
 
-    val = d['query'] if d['type'] == 'SOQL' else ''
-    val = d['method'] if d['type'] == 'METHOD' else val
-    val = d['method'] if d['type'] == 'CONSTRUCTOR' else val
-    val = d['method'] if d['type'] == 'CODE_UNIT' else val
-    val = f"{d['OP']} {d['Type']}" if d['type'] == 'DML' else val
-    val = f"{d['limitType']} " if d['type'] == 'LIMITS' else val
-    val = d['string'] if d['type'] == 'DEBUG' else val
-    val = d['string'] if d['type'] == 'VAR_ASSIGN' else val
+  #  val = d['query'] if d['type'] == 'SOQL' else ''
+  #  val = d['method'] if d['type'] == 'METHOD' else val
+  #  val = d['method'] if d['type'] == 'CONSTRUCTOR' else val
+  #  val = d['method'] if d['type'] == 'CODE_UNIT' else val
+  #  val = f"{d['OP']} {d['Type']}" if d['type'] == 'DML' else val
+  #  val = f"{d['limitType']} " if d['type'] == 'LIMITS' else val
+  #  val = d['string'] if d['type'] == 'DEBUG' else val
+  #  val = d['string'] if d['type'] == 'VAR_ASSIGN' else val
+  #  val = d['string'] if d['type'] == 'NAMED_CRD' else val
+  #  val = d['string'] if d['type'] == 'CALLOUT' else val
+  #  val = d['string'] if d['type'] == 'USER_INFO' else val
+    if 'output' not in d:
+        print()
+    val = d['output']
 
     if val == '':
         print()
@@ -516,26 +632,57 @@ def printParsedLog(d):
 
     _cpuPrevD = _cpuTime0 - int(_context['previousCPUTime'])
 
-    _l = f"{level}"
-    prev = _context['prevTimes'][_l][1] if _l in _context['prevTimes'] else _context['prevTimes'][f"{level-1}"][0]
-    _elapsedPrevD = d['timeStamp'][0] - prev
-    _context['prevTimes'][_l] = [d['timeStamp'][0],_timeStamp1]
+   # _slevel = f"{level}"
+#    if level == 0 and _context['prevTimes'][_slevel][1]==0:
+#        _elapsedPrevD = d['timeStamp'][0]
+
+    if level == _context['prevLevel']:
+        _elapsedPrevD = d['timeStamp'][0] - _context['prevTimes'][level][1]
+
+    if level > _context['prevLevel']:
+        _elapsedPrevD = d['timeStamp'][0] - _context['prevTimes'][_context['prevLevel']][0]
+
+    if level<_context['prevLevel']:
+        _elapsedPrevD = d['timeStamp'][0] - _context['prevTimes'][level][1]
+
+    if _elapsedPrevD <0:
+        if '***' in d['output']:
+            _elapsedPrevD = d['timeStamp'][0] - _context['prevTimes'][level][0]
+
+    _context['prevTimes'][level] = [d['timeStamp'][0],_timeStamp1]
+
+   # prev = _context['prevTimes'][level][1] if level in _context['prevTimes'] else _context['prevTimes'][f"{level-1}"][0]
+   # _elapsedPrevD = d['timeStamp'][0] - prev
+   # _context['prevTimes'][level] = [d['timeStamp'][0],_timeStamp1]
     _elapsedPrevD = ms(_elapsedPrevD)
 
+    _context['prevLevel'] = level
+
     _exp = "!" if d['exception'] == True else ''
 
-    _sql2 = d['SOQLQueries'][1] if len(d['SOQLQueries'])>1 else d['SOQLQueries'][0]
+    _sql2 = d['SOQLQueries'][1] if len(d['SOQLQueries'])>1 else ''
     _sqlcmt2 = d['cmtSOQLQueries'][1] if len(d['cmtSOQLQueries'])>1 else d['cmtSOQLQueries'][0]
 
     _context['previousCPUTime'] = _cpuTime0
     _context['previousElapsedTime']  = d['elapsedTime']
 
-    if _context['firstLineOut'] == True:
-        print(f"{' ':15}|{'time(ms)':10}|{'elapsed':10}|{'time1(ns)':12}|{'time2(ns)':12}|{'ExecTime':10}|{'QueriesStack':15}|{'Qd':4}|{'Qt':4}|{'L':1}|{'cpuD':6}|{'CPUin':6}|{'CPUout':6}|{'Q':3}|{'Qcm':3}|{'type':12}|{'E':1}|{'al':4}{'':50}")
-        _context['firstLineOut'] = False
+    if d['type'] in ['SOQL','DML','VAR_ASSIGN'] and level == 0:
+        _typeColor = CYELLOW 
+    else:
+         _typeColor = ''
 
-    print(f"{i:15} {ms(d['elapsedTime']):10}|{_elapsedPrevD:8}|{d['timeStamp'][0]:12}|{_timeStamp1:12}|{ms(delta(d,'timeStamp')):10}|{_totalQueriesTrace:15}|{_totalQueriesD:4}|{_totalQueries1:4}|{_plimit:1}|{_cpuPrevD:6}|{_cpuTime0:6}|{_cpuTime1:6}|{_sql2:3}|{_sqlcmt2:3}|{_type:12}|{_exp:1}|{_apexline:>4}|{val[:150]:50}"+CEND)
+    if 1==2:
+        if _context['firstLineOut'] == True:
+            print(f"{' ':15}|{'time(ms)':10}|{'elapsed':10}|{'time1(ns)':12}|{'time2(ns)':12}|{'ExecTime':10}|{'QueriesStack':15}|{'Qd':4}|{'Qt':4}|{'L':1}|{'cpuD':6}|{'CPUin':6}|{'CPUout':6}|{'Q':3}|{'Qcm':3}|{'type':12}|{'E':1}|{'al':4}{'':50}")
+            _context['firstLineOut'] = False
 
+        print(f"{i:15} {ms(d['elapsedTime']):10}|{_elapsedPrevD:8}|{d['timeStamp'][0]:12}|{_timeStamp1:12}|{ms(delta(d,'timeStamp')):10}|{_totalQueriesTrace:15}|{_totalQueriesD:4}|{_totalQueries1:4}|{_plimit:1}|{_cpuPrevD:6}|{_cpuTime0:6}|{_cpuTime1:6}|{_sql2:3}|{_sqlcmt2:3}|{_typeColor}{_type:12}{CEND}|{_exp:1}|{_apexline:>4}|{val[:150]:50}"+CEND)
+    else:
+        if _context['firstLineOut'] == True:
+            print(f"{' ':15}|{'time(ms)':10}|{'elapsed':10}|{'time1(ns)':12}|{'time2(ns)':12}|{'t2-t1':10}|{'Qd':4}|{'Qt':4}|{'L':1}|{'cpuD':6}|{'CPUin':6}|{'Q':3}|{'Qcm':3}|{'type':12}|{'E':1}|{'al':4}{'':50}")
+            _context['firstLineOut'] = False
+
+        print(f"{'':15} {ms(d['elapsedTime']):10}|{_elapsedPrevD:8}|{d['timeStamp'][0]:12}|{_timeStamp1:12}|{ms(delta(d,'timeStamp')):10}|{_totalQueriesD:4}|{_totalQueries1:4}|{_plimit:1}|{_cpuPrevD:6}|{_cpuTime0:6}|{_sql2:3}|{_sqlcmt2:3}|{_typeColor}{_type:12}{CEND}|{_exp:1}|{_apexline:>4}|{val[:150]:50}"+CEND)
 
 logsList = []
 def ms(val):
@@ -547,20 +694,39 @@ def printIdent(string,ident):
     print(str + string)
 
 
-def delFromList(theList,field,value):
-    for i in range(len(theList)):
-        obj = theList[len(theList)-i-1]
+def delFromList(theList,field,value,endsWith=False):
+    try:
+        for i,obj in enumerate(theList):
+            if field in obj:
+                if endsWith == True:
+                    if obj[field].endswith(value):
+                        theList.pop(i)
+                        return obj    
+                else:
+                    if obj[field] == value:
+                        theList.pop(i)
+                        return obj
+    except Exception as e:
+        print(e) 
+    return None
 
-        if obj[field] == value:
-            theList.pop(len(theList)-i-1)
-            return obj
+def delFromListX(theList,field,value):
+    try:
+        for i in range(len(theList)):
+            obj = theList[len(theList)-i-1]
 
-    for i in range(len(theList)):
-        obj = theList[len(theList)-i-1]
+            if obj[field] == value:
+                theList.pop(len(theList)-i-1)
+                return obj
 
-        if value in obj[field]:
-            theList.pop(len(theList)-i-1)
-            return obj
+        for i in range(len(theList)):
+            obj = theList[len(theList)-i-1]
 
+            if value in obj[field]:
+                theList.pop(len(theList)-i-1)
+                return obj
+    except Exception as e:
+        print(e)
+        return None
     return None    
 
