@@ -36,6 +36,25 @@ CNEGATIVE = "\033[7m"
 CCROSSED = "\033[9m"
 CEND = "\033[0m"
 #----------------------------------------------------------------------------------------------------
+def stringifyUrlParameters(parameters,exclude,initSeparator='?'):
+  isFirst = True
+
+  ret = ''
+  for name in parameters:
+    if name in exclude:
+      continue
+    if(parameters[name] == None):
+      continue
+    separator = initSeparator if isFirst == True else '&'
+    value = parameters[name]
+    if isinstance(value, str) == False:
+      value = simplejson.dumps(value)
+    paramStr = f'{separator}{name}={value}'
+    ret = ret + paramStr
+    isFirst = False
+
+  return ret
+
 def executeCommand(command):
     list_files = subprocess.run(command,capture_output=True,text=True)
     return list_files
@@ -187,11 +206,12 @@ def deleteNulls(obj,systemFields=True,nulls=True,delAttributes=True):
             res[key] = obj[key]
     return res
 
-def printFormated(records,fieldsString=None):
+def printFormated(records,fieldsString=None,rename=None,exclude=None):
     """
-    - fieldsString: format field-name:field2-name2:field3
+    - fieldsString: format field%name:field2%name2:field3
         field is the field in the obj. Name is how it will be printed (the short version). 
         name3 is equivalent to field3-field3
+    - rename : field%name:field2%name2
     """
     if records is None or len(records) == 0:
         print('No records to print')
@@ -201,20 +221,27 @@ def printFormated(records,fieldsString=None):
     if 'records' in records:
         records = records['records']
 
-    _fields = {}
-
     if fieldsString == None:
         fieldsString = ":".join(records[0].keys())
     fields = fieldsString.split(':')
+    if exclude != None:
+        fields2 = [field for field in fields if field not in exclude.split(':')]
+        fields = fields2
+
+    _renames = {}
+    if rename != None:
+        renames = rename.split(':')
+        for ren in renames:
+            chunks = ren.split('%')
+            _renames[chunks[0]] = chunks[1]
+    
+    _fields = {}
     for field in fields:
-        p = field.split('%') #this was -
         v = {}
-        if len(p)>1:
-            v['name'] = p[1]
-        else:
-            v['name'] = p[0]
-        v['size'] = len(v['name'])
-        _fields[p[0]] = v
+        v['obj_name'] = field
+        v['print_name'] = _renames[field] if field in _renames else field
+        v['size'] = len(v['print_name'])
+        _fields[field] = v
 
     for record in records:
         for key in _fields.keys():
@@ -231,7 +258,7 @@ def printFormated(records,fieldsString=None):
         if key == "__color__":
             continue
         field = _fields[key]        
-        value = field['name']
+        value = field['print_name']
 
         line = f"{line}{value:{int(field['size']+spacing)}}"
     print(line)
@@ -413,4 +440,56 @@ def computeTimes(call,name,times,_type=None):
         print(debugTimes)
 
     print (sec)
-    return debugTimes
\ No newline at end of file
+    return debugTimes
+
+def repeatingSequence(objs,field):
+
+    repeatingSequences = []
+    lastX = 0
+    for x,val in enumerate(objs):
+        if x < lastX:
+            continue
+        firstdelta = None
+        lastDelta = 0
+        positions = []
+
+        for y in range(x+1,len(objs)):
+            if field not in objs[y]:
+                _o = objs[y]
+                print()
+            if objs[y][field] == objs[x][field]:
+                line = objs[y][field]
+                if firstdelta == None:
+                    firstdelta = y-x
+                    lastDelta = firstdelta
+                else:
+                    lastDelta = y - lastX
+                
+                seq = True
+                if lastDelta == firstdelta:
+                    for z in range(0,firstdelta):
+                        if y+z >= len(objs):
+                            break
+                        if objs[y+z][field] != objs[x+z][field]:
+                            seq = False
+                            break
+                    if seq == True:
+                        if len(positions) == 0:
+                            positions.append(y-firstdelta)
+                        positions.append(y)
+                else:
+                    break
+                if seq == True:
+                    lastX = y
+
+        if len(positions) > 0:
+            repeatingSequences.append(positions)
+    #        print(f"{positions}   {firstdelta}" )
+    #        for p in positions:
+    #            for m in range(0,firstdelta):
+
+     #               print(f"{p+m}  {objs[p+m][field]}")
+     #       print()
+
+    return repeatingSequences
+    print(repeatingSequences)
\ No newline at end of file
