@@ -5,7 +5,7 @@ from InCli.SFAPI import utils
 import unittest
 from InCli import InCli
 #from InCli.SFAPI import restClient
-
+import traceback
 class Test_Main(unittest.TestCase):
 
     def test_q(self):
@@ -34,7 +34,7 @@ class Test_Main(unittest.TestCase):
 
     def test_check_Catalog(self):
         try:
-            InCli._main(["-u","NOSQSM","-checkCatalogs"])
+            InCli._main(["-u","NOSQSM","-cc"])
         except Exception as e:
             utils.printException(e)
             print()
@@ -64,7 +64,13 @@ class Test_Main(unittest.TestCase):
         InCli._main(["-u","NOSDEV","-logs"])
 
     def test_log_ID(self):
-        InCli._main(["-u","NOSDEV","-logs","07L3O00000Dazx8UAB"])
+        try:
+            InCli._main(["-u","NOSDEV","-logs","07L3O00000DgHUZUA3"])
+        except Exception as e:
+            print(e)
+            traceback.format_exc()
+    def test_log_file(self):
+        InCli._main(["-u","NOSDEV","-logs","07L3O00000DfytZUAR","-level","0"])
 
     def test_log_last(self):
         InCli._main(["-u","NOSDEV","-logs","-last","10"])
